/*
PRICE was written by Graham Ruby.
This file is part of PRICE.

PRICE is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

PRICE is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with PRICE.  If not, see <http://www.gnu.org/licenses/>.
*/

/* This class separates out the task of mapping reads to existing
 * contigs from the rest of the business of running an ExtendCycle.
 */
#ifndef EXTENDJOBMAPPER_H
#define EXTENDJOBMAPPER_H

#include <set>
#include <map>
#include "ScoredSeq.h"
#include "ScoredSeqWithCarriers.h"
#include "ScoredSeqFlip.h"
#include "ScoredSeqCollectionBwt.h"
#include "ScoredSeqNormalized.h"
#include "ParamsMapping.h"
#include "ExtendCycle.h"
using namespace::std;

class ExtendJobMapper {

 public:
  ExtendJobMapper();
  // no max value
  ExtendJobMapper(set<ScoredSeqWithCarriers*>* contigsWithCarriers, float fractMapId);
  // edgeSize < 0 for no max value
  ExtendJobMapper(set<ScoredSeqWithCarriers*>* contigsWithCarriers, float fractMapId, long edgeSize);
  ~ExtendJobMapper();

  // deep deletes any queries without matches; modifies the query array by
  // placing those that mapped to something at the front of the array and
  // deleting everything with no hits; returns the number of returned queries
  // with hits
  long mapQueries(ScoredSeqWithCarriers** queries, long numQueries);
  //void getUsedReads(set<ScoredSeq*>* usedReads);

 private:

  long _edgeSize;
  int _plusIndex;
  int _minusIndex;
  int _frontIndex;
  int _backIndex;

  // for read mapping
  float _fractMapId;

  // references are Subseq encasing WC encasing Normalized encasing Shallow
  ScoredSeqCollectionBwt* _mappingCollectionFront;
  ScoredSeqCollectionBwt* _mappingCollectionBack;

  void findMatchesHelper(ScoredSeq* currentQuery, char sense, set<ScoredSeqWithCarriers*>* matchingContigs,
			 ScoredSeqCollectionBwt* mappingCollection);
  // here, "side" should be 0 for plus and 1 for minus
  void addToContigMatchesHelper(ScoredSeqWithCarriers* seqToAdd, int side, int frontOrBack, set<ScoredSeqWithCarriers*>* matchingContigs);

};

#endif
