/*
PRICE was written by Graham Ruby.
This file is part of PRICE.

PRICE is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

PRICE is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with PRICE.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef SCOREDSEQSHALLOW_CPP
#define SCOREDSEQSHALLOW_CPP

#include "ScoredSeqShallow.h"
#include <cstring>
using namespace::std;

long ScoredSeqShallow::_readCount = 0;

ScoredSeqShallow::ScoredSeqShallow(){}; //default constructor

void ScoredSeqShallow::OK() {
}

// OVERLOADED CONSTRUCTORS (this goes on for a while)

ScoredSeqShallow::~ScoredSeqShallow(){
  delete [] _seq;
  delete [] _rcSeq;
  delete [] _scores;
  delete [] _links;
  //_readCount--;
}


void ScoredSeqShallow::constructorSeqHelper(string seq){
  _size = seq.size();
  _seq = new char [_size+1];
  strcpy (_seq, seq.c_str());
  _rcSeq = nucleicAcidUtilities::reverseComplement(_seq,_size);
}


ScoredSeqShallow::ScoredSeqShallow(string seq, vector<float>* scores) :
  _pe(0)
{
  constructorSeqHelper(seq);
  long seqSizeMin1 = _size - 1;
  _links = new float[ seqSizeMin1 ];
  _scores = new float[ seqSizeMin1 + 1 ];
  for (long n = 0; n < seqSizeMin1; ++n){
    _links[n] = 1.0;
    _scores[n] = (*scores)[n];
  }
  _scores[ seqSizeMin1 ] = (*scores)[ seqSizeMin1 ];
}


ScoredSeqShallow::ScoredSeqShallow(string seq, vector<float>* scores, vector<float>* links) :
  _pe(0)
{
  constructorSeqHelper(seq);
  long seqSizeMin1 = _size - 1;
  _links = new float[ seqSizeMin1 ];
  _scores = new float[ seqSizeMin1 + 1 ];
  for (long n = 0; n < seqSizeMin1; ++n){
    _links[n] = (*links)[n];
    _scores[n] = (*scores)[n];
  }
  _scores[ seqSizeMin1 ] = (*scores)[ seqSizeMin1 ];
}

// STREAMLINED
ScoredSeqShallow::ScoredSeqShallow(string seq, float* scores) :
  _pe(0)
{
  constructorSeqHelper(seq);
  long seqSizeMin1 = _size - 1;
  _links = new float[ seqSizeMin1 ];
  _scores = new float[ seqSizeMin1 + 1 ];
  for (long n = 0; n < seqSizeMin1; ++n){
    _links[n] = 1.0;
    _scores[n] = scores[n];
  }
  _scores[ seqSizeMin1 ] = scores[ seqSizeMin1 ];
}
ScoredSeqShallow::ScoredSeqShallow(string seq, float* scores, float* links) :
  _pe(0)
{
  constructorSeqHelper(seq);
  long seqSizeMin1 = _size - 1;
  _links = new float[ seqSizeMin1 ];
  _scores = new float[ seqSizeMin1 + 1 ];
  for (long n = 0; n < seqSizeMin1; ++n){
    _links[n] = links[n];
    _scores[n] = scores[n];
  }
  _scores[ seqSizeMin1 ] = scores[ seqSizeMin1 ];
}



// CHAR ARRAYS
ScoredSeqShallow::ScoredSeqShallow(char* seq, float* scores, long size) :
  _size(size),
  _pe(0)
{
  long seqSizeMin1 = _size - 1;
  _seq = new char[_size + 1];
  _seq[_size] = '\0';
  _links = new float[_size - 1];
  _scores = new float[_size];
  for (long n = 0; n < seqSizeMin1; ++n){
    _links[n] = 1.0;
    _scores[n] = scores[n];
    _seq[n] = seq[n];
  }
  _scores[ seqSizeMin1 ] = scores[ seqSizeMin1 ];
  _seq[ seqSizeMin1 ] = seq[ seqSizeMin1 ];
  _rcSeq = nucleicAcidUtilities::reverseComplement(_seq,_size);
}
ScoredSeqShallow::ScoredSeqShallow(char* seq, float* scores, float* links, long size) :
  _size(size),
  _pe(0)
{
  long seqSizeMin1 = _size - 1;
  _seq = new char[_size + 1];
  _seq[_size] = '\0';
  _links = new float[_size - 1];
  _scores = new float[_size];
  for (long n = 0; n < seqSizeMin1; ++n){
    _links[n] = links[n];
    _scores[n] = scores[n];
    _seq[n] = seq[n];
  }
  _scores[ seqSizeMin1 ] = scores[ seqSizeMin1 ];
  _seq[ seqSizeMin1 ] = seq[ seqSizeMin1 ];
  _rcSeq = nucleicAcidUtilities::reverseComplement(_seq,_size);
}




ScoredSeqShallow::ScoredSeqShallow(ScoredSeq *seqToCopy, char sense) {
  // take care of the scores and seq
  constructorSeqHelper(seqToCopy->getSeq(sense));
  long seqSizeMin1 = _size - 1;
  _links = new float[ seqSizeMin1 ];
  _scores = new float[ seqSizeMin1 + 1 ];
  for (long n = 0; n < seqSizeMin1; n++){
    _scores[n] = seqToCopy->scoreAtPosition(n,sense);
    _links[n] = seqToCopy->linkAfterPosition(n,sense);
  }
  _scores[seqSizeMin1] = seqToCopy->scoreAtPosition(seqSizeMin1,sense);

  _pe = 0;
  //_readCount++;
}




// this private constructor is used to prevent an endless loop
ScoredSeqShallow::ScoredSeqShallow(ScoredSeq *seqToCopy, char sense, ScoredSeqShallow *pairedSeq) :
  _pe(pairedSeq)
{
  throw AssemblyException::CallingError("I can't believe you are calling this bullshit constructor, you should have deleted it a long time ago.");
  //OK();
  // take care of the scores and seq
  constructorSeqHelper(seqToCopy->getSeq(sense));
  long seqSizeMin1 = _size - 1;
  _links = new float[ seqSizeMin1 ];
  _scores = new float[ seqSizeMin1 + 1 ];
  for (long n = 0; n < seqSizeMin1; n++){
    _scores[n] = seqToCopy->scoreAtPosition(n,sense);
    _links[n] = seqToCopy->linkAfterPosition(n,sense);
  }
  _scores[seqSizeMin1] = seqToCopy->scoreAtPosition(seqSizeMin1,sense);
  //_readCount++;
}

ScoredSeqShallow::ScoredSeqShallow(const ScoredSeqShallow& r){
  throw AssemblyException::ImplementationError("not implemented");
  //_readCount++;
}

long ScoredSeqShallow::getScorePos(long position, char sense) {
  //OK();
  if (sense=='+'){ return position; }
  else if (sense=='-'){ return _size - position - 1; }
  else { throw AssemblyException::ArgError("sense must be (+) or (-)"); }
}

long ScoredSeqShallow::getLinkPos(long position, char sense) {
  //OK();
  if (sense=='+'){ return position; }
  else if (sense=='-'){ return _size - position - 2; }
  else { throw AssemblyException::ArgError("sense must be (+) or (-)"); }
}


float ScoredSeqShallow::scoreAtPosition(long position, char sense) {
  //OK();
  if (position < 0 or position >= _size){
      throw AssemblyException::LengthError("scoreAtPosition: position is outside of the scope of the sequence");
  }
  return _scores[ getScorePos(position,sense) ];
}

float ScoredSeqShallow::linkAfterPosition(long position, char sense) {
  //OK();
  if (position < 0 or position >= _size - 1){
      throw AssemblyException::LengthError("linkAfterPosition: position is outside of the scope of the sequence");
  }
  return _links[ getLinkPos(position,sense) ];
}


char ScoredSeqShallow::nucAtPosition(long position, char sense) {
  //OK();
  //PROBLEMS: define "length", define a more-efficient method for
  //dealing with sense. define "reverseCompChar" method.
  if (position < 0 or position >= _size){
      throw AssemblyException::LengthError("nucAtPosition: position is outside of the scope of the sequence");
  }
  if (sense=='+'){
     return _seq[ position ];
    }
  else if (sense=='-'){
     return _rcSeq[ position ];
  } else {
    throw AssemblyException::ArgError("sense must be '+' or '-'");
  }
}

string ScoredSeqShallow::getSeq(char sense) {
  //OK();
  if (sense=='+'){
    return string(_seq);
  } else {
    return string(_rcSeq);
  }
}
string ScoredSeqShallow::getSubseq(long position, long length, char sense){
  //OK();
  if (sense=='+'){
    return string(_seq).substr(position,length);
  } else {
    return string(_rcSeq).substr(position,length);
  }
}



long ScoredSeqShallow::size() {
  //OK();
  return _size;
}

void ScoredSeqShallow::buffer(){}
void ScoredSeqShallow::unbuffer(){}
void ScoredSeqShallow::bottomBuffer(){}
void ScoredSeqShallow::deepDelete(){ delete this; }
void ScoredSeqShallow::deepUnbuffer(){} // unbuffer doesn't do anything, so no need to call it


bool ScoredSeqShallow::hasPairedEnd() {
  //OK();
  return _pe != 0;
}

ScoredSeq * ScoredSeqShallow::getPairedEnd() {
  if (! hasPairedEnd()) {
    throw AssemblyException::CallingError("This ScoredSeq doesn't have a paired end.");
  }
  return _pe;
}
ScoredSeq * ScoredSeqShallow::getTempPairedEnd() {
  if (! hasPairedEnd()) {
    throw AssemblyException::CallingError("This ScoredSeq doesn't have a paired end.");
  }
  return _pe;
}

ScoredSeq * ScoredSeqShallow::shallowCopy(){
  //OK();
  return new ScoredSeqShallow(this,'+');
}
ScoredSeq * ScoredSeqShallow::flipCopy(){
  //OK();
  return new ScoredSeqShallow(this,'-');
}


bool ScoredSeqShallow::isNested(){ return false; }


#endif
